# -*- coding: utf-8 -*-
"""
Настройка конфигуратора

(c)2015 ЗАО "РИПАС"
www.ripas.ru
"""
import metarpc
import tempfile
from gui import mainWindow
from RoadCentre import plugins
from PythonQt import QtCore,QtGui
import sys,os

class concon:
    def __init__(self):
        self.app=None
        if sys.platform == 'win32' or sys.platform == 'cygwin':
            app_name="concon.exe"
        else:
            app_name="concon"
        app_dir=QtCore.QDir(QtGui.QApplication.applicationDirPath())
        if app_dir.exists(app_name):
            self.app=app_dir.absoluteFilePath(app_name)
        else:
            app_dir=QtCore.QDir(QtGui.QApplication.applicationDirPath()+'/concon')
        if app_dir.exists(app_name):
            self.app=app_dir.absoluteFilePath(app_name)
        else:
            for path in os.environ.get('PATH','').split(os.pathsep):
                bin_dir=QtCore.QDir(path)
                if bin_dir.exists(app_name):
                    self.app=bin_dir.absoluteFilePath(app_name)
                    break

    def editDprog(self,dev_id,prog_id):
        if not self.app:
            return
        has_metarpc = hasattr(metarpc,'cnfStorage')
        if not has_metarpc and not hasattr(plugins,'JsonRpcClientPlugin'):
            return
        err_title="Ошибка"
        emty_data="Служба хранилища вернула пустые данные"
        if has_metarpc:
            config_data=metarpc.cnfStorage.getConfig(str(dev_id))
            if config_data.startsWith("<Error>"):
                end_index=config_data.indexOf("</Error>")
                if end_index>0:
                    QtGui.QMessageBox.critical(mainWindow,err_title,str(config_data.mid(7,end_index-7)))
                else:
                    showError(emty_data)
                return
        else:
            if 'cnfStorage' not in plugins.JsonRpcClientPlugin.connectedObjects():
                return
            config_data,err_text=self.__invoke_cnfstorage_rpc("app.getConfig",[dev_id])
            if err_text is not None:
                QtGui.QMessageBox.critical(mainWindow,err_title,err_text)
                return
        if not config_data:
            QtGui.QMessageBox.critical(mainWindow,err_title,emty_data)
            return
        prog_file=tempfile.NamedTemporaryFile(delete=False)
        if prog_id>=50:
            if has_metarpc:
                prog_data=metarpc.cnfStorage.getProg(str(dev_id))
                if prog_data.startsWith("<Error>"):
                    end_index=prog_data.indexOf("</Error>")
                    if end_index>0:
                        QtGui.QMessageBox.critical(mainWindow,err_title,str(prog_data.mid(7,end_index-7)))
                    else:
                        showError(emty_data)
                        return
                prog_file.file.write(prog_data.data())
            else:
                prog_data,err_text = self.__invoke_cnfstorage_rpc("app.getProg",[dev_id])
                if err_text is not None:
                    QtGui.QMessageBox.critical(mainWindow,err_title,err_text)
                    return
                prog_file.file.write(prog_data.encode())
        prog_file.close()
        conf_file=tempfile.NamedTemporaryFile(delete=False)
        if has_metarpc:
            conf_file.file.write(config_data.data())
        else:
            conf_file.file.write(QtCore.QByteArray.fromBase64(config_data).data())
        conf_file.close()
        concon_args=["--controller-config",conf_file.name,"--object-id",str(dev_id),"--prog-id",str(prog_id),"--edit-prog",prog_file.name]
        concon=QtCore.QProcess()
        loop=QtCore.QEventLoop()
        def end_loop(code): loop.quit()
        concon.connect('finished(int)',end_loop)
        for w in QtGui.QApplication.topLevelWidgets(): w.enabled=False
        concon.start(self.app,concon_args)
        getattr(loop,'exec')()
        for w in QtGui.QApplication.topLevelWidgets(): w.enabled=True
        dprog_data=open(prog_file.name,'r').read()
        if len(dprog_data)!=0:
            if has_metarpc:
                metarpc.cnfStorage.putDprog(str(dev_id),dprog_data)
            else:
                ans,err_text = self.__invoke_cnfstorage_rpc("app.putDprog2",[dev_id,dprog_data])
                if err_text is not None:
                    QtGui.QMessageBox.critical(mainWindow,err_title,err_text)
        os.remove(conf_file.name)
        os.remove(prog_file.name)

    def start(self,args=[]):
        if not self.app:
            return
        QtCore.QProcess.startDetached(self.app,args)

    def runwithargs(self,args=[]):
        if not self.app:
            return
        proc=QtCore.QProcess()
        proc.start(self.app,args)
        proc.waitForFinished()
        err=proc.readAllStandardError().data().strip()
        out=proc.readAllStandardOutput().data().strip()
        return (proc.exitCode(),out,err)

    def openConfig(self,conf_path):
        if not self.app:
            return
        QtCore.QProcess.startDetached(self.app,[conf_path])
        
    def __invoke_cnfstorage_rpc(self, method, param):
        json_rpc_ans = plugins.JsonRpcClientPlugin.sendCommand("cnfStorage",method,param)
        err_text = None
        if "error" in json_rpc_ans:
            rpc_err = json_rpc_ans["error"]
            err_text = '%d (%s)' % (rpc_err.get("code"),rpc_err.get("message"))
        return json_rpc_ans.get("result"), err_text

if not hasattr(plugins,'concon'):
    plugins.concon=concon()
